@@ -1,7 +1,7 @@
 """AMQP Messaging Framework for Python"""
-VERSION = (0, 10, 5)
+VERSION = (0, 10, 7)
 __version__ = ".".join(map(str, VERSION))
 __author__ = "Ask Solem"
-__contact__ = "askh@opera.com"
+__contact__ = "ask@celeryproject.org"
 __homepage__ = "http://github.com/ask/carrot/"
 __docformat__ = "restructuredtext"
