@@ -82,16 +82,17 @@ class SyncBackend(BaseBackend):
     def queue_purge(self, queue, **kwargs):
         """Discard all messages in the queue. This will delete the messages
         and results in an empty queue."""
-        return self.channel.queue_purge(queue=queue)
+        return self.channel.queue_purge(queue=queue).message_count
 
     def queue_declare(self, queue, durable, exclusive, auto_delete,
-            warn_if_exists=False):
+            warn_if_exists=False, arguments=None):
         """Declare a named queue."""
 
         return self.channel.queue_declare(queue=queue,
                                           durable=durable,
                                           exclusive=exclusive,
-                                          auto_delete=auto_delete)
+                                          auto_delete=auto_delete,
+                                          arguments=arguments)
 
     def exchange_declare(self, exchange, type, durable, auto_delete):
         """Declare an named exchange."""
@@ -100,8 +101,10 @@ class SyncBackend(BaseBackend):
                                              durable=durable,
                                              auto_delete=auto_delete)
 
-    def queue_bind(self, queue, exchange, routing_key, arguments=None):
+    def queue_bind(self, queue, exchange, routing_key, arguments={}):
         """Bind queue to an exchange using a routing key."""
+        if not arguments:
+            arguments = {}
         return self.channel.queue_bind(queue=queue,
                                        exchange=exchange,
                                        routing_key=routing_key,
@@ -119,7 +122,7 @@ class SyncBackend(BaseBackend):
             there was no messages waiting on the queue.
 
         """
-        raw_message = self.channel.basic_get(queue, no_ack=no_ack)
+        raw_message = self.channel.basic_get(queue=queue, no_ack=no_ack)
         if not raw_message:
             return None
         return self.message_to_python(raw_message)
